/*
 * d_getdim.c
 *
 * $Source: /source/MASTER/garden/datlib/RCS/d_getdim.c,v $
 * $Date: 22/.1/.2 .2:.4:.0 $
 * $Author: holtzman $
 * $Locker:  $
 *
 * This file is part of the dat library.
 *
 * $Log:	d_getdim.c,v $
 * Revision 1.5  22/.1/.2  .2:.4:.0  holtzman
 * Undid a const which wasn't
 * 
 * Revision 1.4  91/11/02  23:31:21  klee
 * updated for const declarations
 * 
 * Revision 1.3  90/12/08  03:17:17  kaboom
 * Replaced under RCS, moved copyright message to copyright.h.
 * 
 * Revision 1.2  89/01/13  02:43:49  jwang
 * Add M.I.T. copyright message.
 * 
 * Revision 1.1  85/06/29  00:09:00  musicus
 * Initial revision
 * 
 * 
 * #include <dat.h>
 * 
 * int d_getdim(d,&nsets,&nchan,&dim,maxdim)
 * 
 *    Reads the DATASETS, CHANNELS and DIMENSIONS keywords from DATFILE d,
 *    and returns the number of DATASETS in nsets (long), CHANNELS in channels
 *    (long), and the DIMENSIONS of the file in the (long) array dim[].
 *    Up to maxdim sizes will be put into the dim[] array; if the number of
 *    dimensions    is larger than this, the last maxdim of these will be put
 *    into dim[] (these are the fastest varying dimensions.)  The total number
 *    of dimensions is returned.
 * 
 *    Inputs:
 *        d - DATFILE pointer
 *        maxdim - maximum number of dimensions we are interested in
 *        dim[] - an empty long array of length maxdim
 *    Outputs:
 *        nsets - number of datasets in file (default=1)
 *        nchan - number of channels per dataset (default=1)
 *        dim[] - the size of each dimension of each channel in each dataset
 *                of the dat file
 *    Return value:
 *        number of dimensions of each channel in the dat file
 * 
 *    Maximum number of dimensions presently set at 10
 */

static char rcsid[] = "$Header: /source/MASTER/garden/datlib/RCS/d_getdim.c,v 1.5 22/.1/.2 .2:.4:.0 holtzman Exp $" ;

#include "copyright.h"
#include <stdio.h>
#include "dat.h"

int d_getdim(d,nrec,nchan,dim,maxdim)
const DATFILE *d;
long *nrec,*nchan;
long dim[];
int maxdim;
{
	int ndim;
	long n[10];
	const char *p;
	int i;

	if(nrec)
	{
		p = d_getkey(d,DATASETS);
		if(p == NULL || sscanf(p,"%ld",nrec)!=1)
			*nrec = 1L;
	}
	if(nchan)
	{
		p = d_getkey(d,CHANNELS);
		if(p == NULL || sscanf(p,"%ld",nchan)!=1)
			*nchan = 1L;
	}
	p = d_getkey(d,DIMENSIONS);
	if(p == NULL)
		return(0);
	ndim = sscanf(p,"%ld %ld %ld %ld %ld %ld %ld %ld %ld %ld",
			&n[0],&n[1],&n[2],&n[3],&n[4],&n[5],&n[6],&n[7],&n[8],&n[9]);
	if(ndim < 0)
		return(0);
	if(maxdim <= 0 || dim == NULL)
		return(ndim);
	if(ndim <= maxdim)
	{
		for(i=0 ; i<ndim ; i++)
			dim[i] = n[i];
	}
	else
	{
		for(i=0 ; i<maxdim ; i++)
			dim[i] = n[i+ndim-maxdim];
	}
	return(ndim);
}
