/*
 * d_setkey.c
 *
 * $Source: /source/MASTER/garden/datlib/RCS/d_setkey.c,v $
 * $Date: 91/11/02 23:31:29 $
 * $Author: klee $
 * $Locker:  $
 *
 * This file is part of the dat library.
 *
 * $Log:	d_setkey.c,v $
 * Revision 1.4  91/11/02  23:31:29  klee
 * updated for const declarations
 * 
 * Revision 1.3  90/12/08  03:56:39  kaboom
 * Replaced under RCS, moved copyright message to copyright.h.
 * 
 * Revision 1.2  89/01/13  02:44:05  jwang
 * Add M.I.T. copyright message.
 * 
 * Revision 1.1  85/06/28  23:50:48  musicus
 * Initial revision
 * 
 *
 * int d_setkey(d,key,value)
 *   DATFILE *d;
 *   char *key, *value;
 * 
 * Makes the value of key in the dat file d header be value.  Create key if 
 * necessary.
 *
 * int d_appkey(d,key,value)
 *   DATFILE *d;
 *   char *key, *value;
 *
 * Appends "value" to the value of "key" in the dat file "d" header.
 * Creates key if necessary.
 *
 */

static char rcsid[] = "$Header: /source/MASTER/garden/datlib/RCS/d_setkey.c,v 1.4 91/11/02 23:31:29 klee Exp $" ;

#include "copyright.h"
#include <stdio.h>
#include <ctype.h>
#include <sys/types.h>
#include <sys/stat.h>
#include "dat.h"

char *calloc(),*realloc(),*malloc();
/* int free(); */

#define max(x, y)	(((x) < (y)) ? (y) : (x))
#define mabs(x)		((x < 0) ? -(x) : (x))
#define min(x, y)	(((x) < (y)) ? (x) : (y))

static int chgkey();


/* d_setkey(d,key,value)
 *
 *	makes the value of skey in header of dat file be value.  Create skey
 *	if necessary.  Returns -1 if failure, NULL if success.
 */

int
d_setkey(d,skey,svalue)
	DATFILE *d;
	const char *skey;
        const char *svalue;
{
	return(chgkey(d,skey,svalue,0,"d_setkey"));
}

/* d_appkey(d,key,value)
 *
 *	Appends "value" to the value of "key" in the dat file "d" header.
 *	Creates key if necessary.
 *
*/
	
int
d_appkey(d,skey,svalue)
	DATFILE *d;
	const char *skey;
        const char *svalue;
{
	return(chgkey(d,skey,svalue,1,"d_appkey"));
}


/* chgkey(d,skey,svalue,flag,name)
 *
 *	Does the work for d_setkey() and d_appkey()
*/

static int
chgkey(d,skey,svalue,flag,name)
	DATFILE *d;
	const char *skey, *svalue, *name;
	int flag;
{
	struct d_token *tp,*tpn;
	char *p;
	
	if((tp=d->db_h) == (struct d_token*) -1)
	{
		fprintf(stderr,"%s: No header file\n",name);
		return(-1) ;
	}

	for(; tp ; tp=tp->d_next)
	{
		if (strcmp(skey,tp->d_key) == 0)
		{
			if(flag==0)
			{
				if((p = realloc(tp->d_value,strlen(svalue)+1)) == NULL)
				{
					perror(name);
					return(-1);
				}
				strcpy(p,svalue);
				tp->d_value = p;
			}
			else
			{
				if((p = realloc(tp->d_value,strlen(tp->d_value)+
						strlen(svalue)+1)) == NULL)
				{
					perror(name);
					return(-1);
				}
				strcat(p,svalue);
				tp->d_value = p;
			}
			d->db_flag |= _DB_WRHDR;
			return(NULL);
		}
		if(tp->d_next == NULL)
			break;
	}
	if((tpn = (struct d_token*) calloc(1,sizeof(struct d_token))) == NULL)
	{
		perror(name);
		return(-1);
	}
	if((tpn->d_key = malloc((unsigned)(strlen(skey)+1))) == NULL)
	{
		perror(name);
		free((char*)tpn);
		return(-1);
	}
	strcpy(tpn->d_key,skey);
	if((tpn->d_value = malloc((unsigned)(strlen(svalue)+1))) == NULL)
	{
		perror(name);
		free((char*)tpn->d_key);
		free((char*)tpn);
		return(-1);
	}
	strcpy(tpn->d_value,svalue);
	if(tp==NULL)
	{
		d->db_h = tpn;
	}
	else
	{
		tp->d_next = tpn;
	}
	d->db_flag |= _DB_WRHDR;
	return(NULL);
}
