/*
 *	   	  Datlib Run-Time Library Reference 
 *		   (and function declaration file)
 *
 *
 *  6/27/91  rune  (port to IBM PS/2) 
 *  7/15/91        (envpath (all following changes for IBM PS/2 only))
 *  7/17/91        (non-static-ing of parsehdr routines)
 *  7/22/91        (_dsize, reverse_convert)
 *  7/25/91        (altconvert, more descriptions)
 *  9/28/91        (daterrh, d_seek, d_open "w" fixed)
 *
 *  Home:
 *      IBMPS2 mods and the routines reverse_convert, altconvert, envpath
 *      currently live only at:
 *
 *             ASGARD:c:\usr\prune\src\lib\src\datlib
 *
 *      The established directory however is:
 *
 *             LADY-DAY:\source\MASTER\garden\datlib
 *
 *  Types:  
 *	DATFILE
 *
 *  Functions (some static):  
 *	addch, altconvert, chgkey, convert, d_appkey, d_appkey_strvec,
 * 	d_cleanup, d_close, d_close_subfile, d_copyhdr,
 *	d_delkey, d_descr, d_flush, d_fork,
 *	d_get_byte_order, d_getdim, d_get_float_rep,
 *	d_getkey, d_getkey_elem, d_getmatrix, d_getsubdim,
 * 	d_gettype, d_ispath, d_open, d_open_subfile,
 *	d_path, d_prhdr, d_rmdat, d_seek, d_set_byte_order,
 *      d_set_float_rep, d_setdim, d_setkey, d_settype,
 *	d_submatrix, d_tell, _daterrh, _dclrinfo, _dmread,
 *	_dmwrite, _doscan, _dread, _dsize, dumphdr, _dwrite, dxopen, envpath,
 *	exit, _getccl, getrange, init_dinfo, _innum, _instr, loadhdr, 
 *	parsehdr, reopenmode, reverse_convert, samesize, subsample, upsample,
 *	writerr
 *
 *
 * DATFILE 
 *
 * source:  dat.h
 *
 * A DATFILE struct has the following form:
 *    DATFILE
 *    {
 *    	struct d_token *db_h;		(ptr to token tree)
 *    	FILE *db_fp;			(FILE pointer for data
 *    	char *db_name;			(datfile name specified in call)
 *    	char db_flag;			(status flags such as _DB_WRHDR)
 *    	char db_nbs;			(log of number of bytes per sample)
 *    };
 * 	db_h normally points to a linked list of token structs, each pointing
 *	  to key and value strings.  If the header is empty or descriptor file
 *	  could not be found, db_h=0.  If the datfile is actually an ordinary
 *	  file and there can be no header, then db_h=-1.  ALL TOKEN STRUCTS
 *	  AND KEYWORD,VALUE STRINGS MUST BE MALLOC'ED FROM THE FREE LIST.
 *	  Flag bit _DB_WRHDR must be set if header is modified.
 *	db_fp is the file pointer for the opened data file.  If no data file
 *	  is open, db_fp=0
 *	db_name is the name of the dat file.  THIS MUST BE MALLOC'ED FROM THE
 *	  FREE LIST.
 *	db_flag has miscellaneous flags.
 *	  _DB_WRHDR indicates header modified in core.
 *	  _DB_GOTTYPE indicates db_nbs is meaningful.
 *	db_nbs is number places to shift sample count left to get byte count.
 *	  This is set by first d_read() or d_write().
 */


int addch(char** buf,char** q, unsigned int* buflen);
/*
 * source:  dat.c 
 *
 * Expands buffer buf by a chunk of PARSEBUF chars
 * Returns -1 if malloc error, 0 if OK  
 * buflen is incremented by size of chunk, buf is repointed to the new
 * buffer, and q is pointed to the start of the newly added on chunk 
 */

#ifdef IBMPS2 
char* altconvert(char *obuf, int otype, char *ibuf, int itype, 
	register int n);
/*
 * source:  convert.c
 *
 * convert between any of data types buUcdDxX
 * maintains machine byte order (is portable)
 * for each of n data of type itype in ibuf writes over a zero as 
 * many bytes of data from least significant to most significant as 
 * will fit as specified by otype into each data in obuf.  Returns obuf.
 */
#endif

void convert(char obuf[], int otype, char ibuf[], int itype, register int n);
/*
 * source:  convert.c
 *
 *      convert between any of data types buUcdDfFzZxX.
 * 	(n is count; gets destroyed)
 *	May be used independent of datfiles.
 *	The following rules are followed:
 *	Real->complex	puts 0 in imaginary part
 *	Complex->real	retains only real part
 *	Float->integer	rounds to nearest integer
 *	Any->integer	clips if out of range
 *	The fact that ibuf is copied to obuf in order of increasing addresses
 *	should be observed if these buffers overlap in memory. 
 */


int d_appkey(DATFILE* d, char *skey, char* svalue);
/* 
 * source: d_setkey.c
 *
 *	Appends "value" to the value of "key" in the dat file "d" header.
 *	Creates key if necessary.
 *      (Modifies d->db_h only)
 */


int d_appkey_strvec(DATFILE* datfile, char* key, int vecsize, char* strvec[]);
/*
 * source: datext.c
 *
 * appends each string in strvec[] seperated by spaces.
 * example:  d_appkey_strvec(d, HISTORY, argc, argv) 
 */


d_cleanup();
/*
 * source:  dat.c
 *
 * flush all datfiles, usually called from libdat.a version of exit() 
 */


int d_close(DATFILE *d);
/*
 * source:  dat.c
 *
 * 	the equivalent of fclose().  Dumps the header into the descriptor
 *	file if the header was modified (db_flag bit _DB_WRHDR set), then
 *	fclose() the data file pointer, and frees all malloc() storage used in
 *	the DATFILE struct, the name, and the token list.
 */


int d_close_subfile(DATFILE* datfile);
/* 
 * source: datext.c
 *
 *  total code:  return(d_close(datfile));
 */


int d_copyhdr(DATFILE* dout, DATFILE* din);
/*
 * source: copyhd.c
 *
 *    Copies the header from the open dat file "din" to the open dat file
 *    "dout".  Note that this routine only affects
 *    the in-core version of the header, and does not modify any files.
 *    d_close(dout) will have to be called to make the header permanent.
 * 
 *    Inputs:
 *       din - DATFILE pointer for input dat file whose header is to be copied
 *       dout - DATFILE pointer for output dat file getting new header
 *    Returns value:
 *       0 if OK
 *       -1 if malloc() failed
 */


int d_delkey(DATFILE* d, char* key);
/* 
 * source: d_delkey.c
 *
 * 	deletes the key from the dat file header
 *	returns NULL if succeeds, -1 if can't find the key
 *	if succeeds, sets _DB_WRHDR flag to indicate header modified.
 */


DATFILE* d_descr(char* name);
/*
 * source:  dat.c
 *
 *    Like d_open, but only reads in the header from the data file, and does
 *    not open the data file.  Returns a struct of type DATFILE with a pointer
 *    to the parsed token list, a null file pointer, and the name of
 *    the datfile.  If "name" isn't a directory, or if descriptor file can't
 *    be read, returns NULL.
 */


int d_flush(DATFILE *d);
/*
 * source:  dat.c
 *
 *	the equivalent of fflush().  Dumps the header into the descriptor file
 *	if the header has been modified, and clears the modified bit.  Flushes
 *	the buffered output file.
 */


int d_fork(DATFILE* in, DATFILE* out[], int mode);
/*
 * source: d_fork.c
 *
 * d_fork splits the DATFILE stream in into a vector of streams out[], each
 * restricted to a subset of the current submatrix of the parent.  Mode, 
 * which is an integer between 0 and ndims-1 or one of the constants
 * D_ADDR_CHAN (-1) or D_ADDR_DSET (-2), selects the dimension to split.
 * For example, use D_ADDR_CHAN to split a color image sequence into
 * three streams, one channel per color separation.
 * The parent stream will be flushed but is otherwise unchanged by d_fork.  
 * Each child will have an I/O pointer independent from those of other 
 * streams.  The number of children is returned.  
 */


enum byte_order d_get_byte_order(DATFILE* d);
/*
 * source: d_arch.c 
 *
 * Returns either MSB, LSB, or NOKEY (include dat.h for those defs)
 * (should be compared with MACH_BYTE_ORDER as is done in d_open).  
 * The byte order determines in what order multi-byte data types 
 * store their bytes.
 */


int d_getdim(DATFILE* d, long* nrec, long* nchan, long dim[], int maxdim);
/*
 * source: d_getdim.c
 * 
 *    Reads the DATASETS, CHANNELS and DIMENSIONS keywords from DATFILE d,
 *    and returns the number of DATASETS in nsets (long), CHANNELS in channels
 *    (long), and the DIMENSIONS of the file in the (long) array dim[].
 *    Up to maxdim sizes will be put into the dim[] array; if the number of
 *    dimensions    is larger than this, the last maxdim of these will be put
 *    into dim[] (these are the fastest varying dimensions.)  The total number
 *    of dimensions is returned.
 * 
 *    Inputs:
 *        d - DATFILE pointer
 *        maxdim - maximum number of dimensions we are interested in
 *        dim[] - an empty long array of length maxdim
 *    Outputs:
 *        nsets - number of datasets in file (default=1)
 *        nchan - number of channels per dataset (default=1)
 *        dim[] - the size of each dimension of each channel in each dataset
 *                of the dat file
 *    Return value:
 *        number of dimensions of each channel in the dat file
 * 
 *    Maximum number of dimensions presently set at 10
 */


enum float_rep d_get_float_rep(DATFILE* d);
/*
 * source: d_arch.c
 *
 * Returns IEEE_FLOAT, VAX_FLOAT, or NOKEY (should be compared with
 * MACH_FLOAT_REP as is done in d_open).
 */


char* d_getkey(DATFILE* d, char* gkey);
/*
 * source: d_getkey.c
 *
 *	returns the value of key from header of dat file, or else NULL
 *      (searches d->db_h)
 */


char* d_getkey_elem(DATFILE* datfile, char* keyword, long index);
/*
 * source: datext.c
 *
 * SIDE EFFECTS: Opens dat subfiles. (no it doesn't!  it calls d_getkey
 * which looks through datfile->db_h.  no files are opened at all?!?)
 *
 * gets the (index)th (first is zero) word in the value associated
 * with keyword (assumes each word is seperated by at least one space).
 */


int d_getmatrix(DATFILE* df, struct d_addr* start, struct d_addr* stop,
	struct d_addr *step);
/*
 * source: d_getmat.c
 *
 * struct d_addr *start,*stop,*step may be NULL if not needed by caller 
 * Get definition of current submatrix (whole datfile if
 * submatrix not in effect).  Returns number of dimensions,
 * or -1 if unknown 
 * More logically d_getsubmatrix, but that name is too long on some machines
 */


int d_getsubdim(DATFILE* df, struct d_addr* size);
/*
 * source: d_getsub.c
 *
 * Get dimensions of current submatrix (whole datfile if
 * submatrix not in effect).  Returns number of dimensions,
 *or -1 if unknown 
 */


int d_gettype(DATFILE* d, int* nbps);
/*
 * source: d_gettyp.c
 *
 *    Reads value of the "type" keyword from the DATFILE d, looks it up
 *    in a table of known types and returns the appropriate code.  If
 *    type is unknown or type keyword is missing, then returns 0.
 *    If &nbps argument is non-zero, sets the integer variable nbps to
 *    the number of bytes per sample for the given data type.
 * 
 *    Inputs:
 *        d - DATFILE pointer
 *    Outputs:
 *        nbps - address of integer variable where will store the number of
 *               bytes per sample for this data type
 *    Return value:
 *        code for the type of the dat file (see below)
 *        0 if type not found or doesn't exist, sets nbps to 1.
 *
 * static struct typ stype[] = {
 *	{ASCII,			'a',		sizeof(char)},
 *	{VARIANT,		'v',		1},
 *	{INTEGER1,		'c',		1},
 *	{INTEGER2,		'd',		2},
 *	{INTEGER4,		'D',		4},
 *	{UNSIGNED1,		'b',		1},
 *	{UNSIGNED2,		'u',		2},
 *	{UNSIGNED4,		'U',		4},
 *	{FLOAT,			'f',		sizeof(float)},
 *	{DOUBLE,		'F',		sizeof(double)},
 *	{COMPLEX,		'z',		2*sizeof(float)},
 *	{DCOMPLEX,		'Z',		2*sizeof(double)},
 *	{FIX2,			'x',		2},
 *	{FIX4,			'X',		4},
 *	{(char*)NULL,		 0,		0}
 * } ;
 */


int d_ispath(char *s);
/*
 * source: picpath.c
 *
 * look for a '/' (or '\\' if IBMPS2 def) in a string.  
 * if it's there, assume we are dealing with a
 * path.
 *
 * (not strictly accurate; it may be a subpath like "sys/junk":
 *  on an ordinary IBM, the way to do it is to check to see if either
 *  the first character is a PATHCHAR (\ or /) or the second character
 *  is a colon (:).  Otherwise it should be assumed to be a subpath)
 */


DATFILE* d_open(char* name, char* mode);
/*
 * source:  dat.c
 *
 *   is something like fopen().  It opens a datfile with access mode
 *   specified by the string mode.  (This may be "r","w","a","r+","w+","a+").
 *   If mode="w":
 *	if name = old datfile name, or ordinary filename, deletes old file.
 *	if name = special file, creates simple output file, sets db_h = -1
 *	if name = other directory, then error.
 *	if name not exist, creates new dat file.
 *   If mode="r":
 *	if name not exist, then error.
 *	if name = old datfile, load header in descriptor file, open
 *		data file for buffered I/O, return struct of type DATFILE
 *	if name = old filename, open file for buffered I/O, return struct
 *		of type DATFILE with db_h = -1
 *   If mode="a":
 *	if name not exist, create new dat file.
 *	if name old datfile, load header, open data file for append
 *	if name old filename, open file for append, set db_h = -1
 *   If error occurs, return NULL.
 *
 * all errors print error messages.
 *
 * also checks byte order and float rep compatibility if applicable,
 * and exits if incompatible. ifdef IBMPS2, doesn't exit just returns NULL.
 *
 * ifdef IBMPS2, tacks on a "b" to the mode before fopen is called.
 */


DATFILE* d_open_subfile(DATFILE* datfile, char* subname, char* mode);
/*
 * source: datext.c
 *
 * does the string manipulations implied by
 * return(d_open(d_getname(datfile)"/"subname, mode))
 */


int d_path(char* picname, char* picturepath);
/*
 * source: picpath.c
 *
 *  d_path will take a "picture" name-- that is, a string like "title" or
 *  "/u/pix/pam" and return an integer representing the existence and/or
 *  format of the data.
 *  
 *  d_open will, given a simple picture title without a specified path,
 *  search through possible paths for that picture, or else it will search
 *  for the picture through the path specified, if any.
 *  
 *  If a path is specified,
 *  * the integer 1 is returned if the picture
 *  exists there,
 *
 *  If no path is specified,
 *  * the integer 1 is returned if path is found
 *  to be: "/u/pix/title/data", e.g. a DAT file
 *
 *  * 0 (zero) is returned if it does not exist in either case. 
 *
 *  picopen looks at the environment variable PICPATH to determine where
 *  to open the picture file.  You should have something along the lines
 *  of `setenv PICPATH ( /u/pix . /u/cm )' in your .cshrc file on the
 *  SUNs, and perhaps `setenv PICPATH ( /suns/u/pix /pix/suns . )' on the
 *  VAX.
 *  
 *  When it cannot open a file (for whatever reason), picopen returns a
 *  negative number.
 *  
 * (see envpath; IBM version is:  envpath(picname, PICPATH, picturepath);
 */


d_prhdr(DATFILE* d, FILE* fp);
/*
 * source: d_prhdr.c
 *
 *	prints in a nice form the descriptors of the dat file d onto buffered
 *	file pointer fp
 */


int d_rmdat(char name[]);
/*
 * source: d_rmdat.c
 *
 * remove datfile called "name" (also removes normal files)
 */


long d_seek(DATFILE* df, struct d_addr* distance, int whence);
/*
 * source: d_seek.c
 *
 * don't allow seeking to EOF position 
 * IBM PS/2 MSC:  whence one of { 0 (SEEK_SET), 1 (SEEK_CUR), 2 (SEEK_END) }
 */


int d_set_byte_order(DATFILE* d, enum byte_order bo);
/*
 * source: d_arch.c
 *
 * returns 0 on success, -1 on failure
 */


int d_set_float_rep(DATFILE* d, enum float_rep fr);
/*
 * source: d_arch.c
 */


int d_setdim(DATFILE* d, long nset, long nchan, long xy[], int ndim);
/*
 * source: d_setdim.c
 *
 * Sets the DATASETS, CHANNELS and DIMENSIONS keywords in a dat file d
 * If nset>0, uses this as DATASETS value
 * If nchan>0, uses this as CHANNELS value
 * If ndim>0, uses first ndim entries in xy[] as DIMENSIONS
 * Returns 0 if success, -1 if failure
 */


int d_setkey(DATFILE* d, char* skey, char* svalue);
/* 
 * source: d_setkey.c
 *
 *	makes the value of skey in header of dat file be value.  Create skey
 *	if necessary.  Returns -1 if failure, NULL if success.
 */


int d_settype(DATFILE* d, int type);
/*
 * source: d_gettyp.c
 *
 *    Looks up the single character code "type" in the table below, then
 *    sets the TYPE keyword in DATFILE "d" to the corresponding string value.
 *    (see d_gettype() for type table)
 *
 *    Inputs:
 *        d - DATFILE pointer
 *        type - single character type code
 *    Outputs:
 *        d_setkey() on TYPE keyword
 *    Return value:
 *        0 if finds type code in table
 *        -1 if not find type code in table
 */


long d_submatrix(DATFILE* df, struct d_addr* start, struct d_addr* stop, 
	struct d_addr* step);
/*
 * source: d_submat.c
 */

		
int d_tell(DATFILE* df, struct d_addr* where);
/*
 * source: d_tell.c
 *
 * Get position in current submatrix.  Returns -1 if past end of data. 
 */


#ifndef IBMPS2
 		/* illegal prototype:
int _daterrh(fmt, arg);
/*
 * source:  daterrh.c
 *
 * a datfile i/o error handler 
 */
#else
int _daterrh(char* fmt, char* arg);
/*
 * source:  daterrh.c
 *
 * does fprintf(stderr, fmt, arg) and exit(-1) 
 */
#endif


_dclrinfo(int fd);
/*
 * source: datio.c
 *
 * defined is struct dinfo *_Dinfo[_NFILE], initialized with NULLI's 
 * reset _Dinfo[fd] entry; must be called before a previously used
 * file descriptor is reused for a newly opened datfile.  Should be
 * called from d_close. 
 */


int _dmread(DATFILE* df, char** bufp, int n, int buftype);
/*
 * source: dmread.c
 */


int _dmwrite(DATFILE* df, char* buf, int n, int buftype);
/*
 * source: dmwrite.c
 *
 * does a return(_dwrite(df, buf, n, buftype)) and free(buf)
 */


_doscan(struct _iobuf* iop, register char* fmt, register int** argp);
/*
 * source: doscan.c
 *
 * WARNING:  does not malloc everything
 */


int _dread(DATFILE* df, char* buf, int n, int cvt);
/*
 * source: dread.c
 */

int _dsize(int dtype);
/*
 * source: convert.c
 *
 * given a data type, return log2 bytes/sample 
 */

#ifdef IBMPS2 /* this function is normally static */
int dumphdr(DATFILE *d);
/*
 * source: dat.c
 *
 *	writes the token list in d->db_h to the file d->db_name/DESCRIPTOR.
 *	Returns -1 if error.
 */
#endif

int _dwrite(DATFILE* df, char* buf, int n, int cvt);
/*
 * source: dwrite.c
 */


DATFILE* dxopen(char* fname, char* mode);
/*
 * source: dxopen.c
 *
 * dxopen - extended datfile open
 *
 * The programming needed to set up subarrays can be tedious.  dxopen
 * automatically parses "filename[range_spec]" and generates calls to
 * d_open and d_submatrix.  If dxopen is substituted for d_open, d_read for
 * read, and d_getsubdim for d_getdim, subarray/subsampled I/O becomes
 * virtually transparent to the program.
 *
 * fname is a datfile name possibly suffixed by a bracketed list
 * of ranges separated by blank or comma or any combination:
 *	[range range ...]
 * or	[range,range,...]
 * or	[range][range]...
 * The syntax of range is defined in getrange(). If ranges are not given, 
 * dxopen
 * behaves just like d_open.  If ranges are given, then fname must already
 * exist and will not be deleted even if mode "w" is selected; the appropriate
 * submatrix will be set up from the range_spec.
 *
 * Limitations: no provision for DELETEDIM.
 *
 * If compiled with -DNORETURN, does exit(-1) instead of return(NULL)
 * upon error.
 */


#ifdef IBMPS2
int envpath(char *inpath, char *env_var, char *outpath); 
/* 
 * source: picpath.c
 *
 *           Searches for the existence of the file specified by 
 *           inpath as it appears; if not there, changes the
 *           search path to those defined in the environmental
 *           variable env_var and searches there (in order from
 *           left to right).  Returns with outpath set to
 *           the full path name of where the file was found.
 *           If the file was found, envpath returns 1, otherwise
 *           it returns 0 and outpath is just set to inpath.
 */
#endif

 		/* multiply defined prototype:
exit(int r0);
/*
 * source: d_cuexit.c
 *
 * Closes all datfiles and files. This routine resides in libdat.a and
 * replaces the standard exit() in libc.a when libdat is loaded before libc.
 */


int getrange(char* program, char* arg, long* start, long* stop, long max, 
	long* step);
/*
 * source: getrange.c
 *
 * Parse a string for range arguments.  Arguments must be of the form:
 *	[start[:stop | +size]][/step]
 * On entry:
 *	program = program name
 *	arg	= string to parse
 *	*start	= default starting value
 *	*stop	= default stopping value
 *	*step	= default step value
 *	max	= maximum value for *start and *stop
 * On exit:
 *	*start 	= parsed value, if any
 *	*stop 	= parsed value, if any
 *	*step 	= parsed value, if any
 *	getrange= STARTSTOP if argument is in the form a:b
 *		= STARTSIZE if argument is in the form a+b
 *		= DELETEDIM if argument is in the form a
 *
 * Note:
 *	The "stop" value returned is one larger than that parsed.  If the
 * user specifies a:b, getrange returns with *start = a and *stop = b+1
 * so that getrange() may be used with d_submatrix() without adjusting 
 * any values.
 */


#ifdef IBMPS2
init_dinfo();
  /*
   * source: datio.c
   *
   * Mallocs _Dinfo structure and sets all entries to NULL.
   * Should be called before beginning any datfile operations
   * (it's called from D_OPEN, which usually is the first datfile
   * operation that occurs).
   */
#endif

#ifdef IBMPS2 /* this function is normally static */
struct d_token* loadhdr(char *name, char *mode);
/*
 * source: dat.c
 *
 *	takes the string passed as the argument, which should be the name
 *	of the datfile directory, and opens up the file DESCRIPTOR within
 *	it, and parses the token list it finds there.  It returns a pointer
 *	to a linked list of tokens.  If mode is "r" and can't find DESCRIPTOR,
 *   returns -1.  If mode is "w" or "a" and can't find DESCRIPTOR, creates
 *   an empty descriptor file and returns NULL.  If error, returns -1.
 *   loadhdr() should only be called if we are really working on a dat file
 *   structure.
 */
#endif

#ifdef IBMPS2 /* this function is normally static */
struct d_token* parsehdr(FILE* fp);
/* 
 * source: dat.c
 *
 *	Actually parses the header, and returns a pointer to the token
 *	list.  It wants a file pointer opened to the header.  Calls malloc()
 *	to allocate each key and value string, and each token struct.
 */
#endif

#ifdef IBMPS2
char* reverse_convert(char *obuf, int otype, char *ibuf, int itype, 
	register int n);
/*
 * source: convert.c
 *
 *  reverse_convert() does the a similar thing as convert() except that it 
 *  reads from the input buffer values with bytes in reverse order from
 *  the way they normally would (for porting data created on alien machines
 *  with different byte orders).  The code for reverse_convert() is 
 *  machine independent (it uses masks).  Just copies as many bytes from
 *  least significant to most significant (machine independent) into the dest,
 *  truncating the rest, putting zeroes in the bytes that obuf has that ibuf
 *  doesn't have.  does types up to four bytes long (no floats).
 *  Returns obuf.
 */
#endif

int samesize(DATFILE* d1, DATFILE* d2);
/*
 * source: samesize.c
 *
 * Returns 1 if d1 and d2 are the same size else returns 0.
 */


subsample(register char outptr[], register char inptr[], int nbps, 
	int h_factor, int h_samples);
/* 
 * source: subsampl.c
 *
 * horizontal subsampler; stolen from /unips/src/pixutil/subsample.c 
 */


upsample(register char outptr[], register char inptr[], int nbps, 
	int h_factor, int h_size);
/*
 * source: upsample.c
 *
 * horizontal upsampler; stolen from /unips/src/pixutil/upsample.c 
 */

 			/* illegal prototype:
writerr(nwritten, file, buffer, count);
/*
 * source: writerr.c
 */
