/*
 * audIOmac.c
 *
 * Play sounds through the mac sound manager using the DoubleBuffer
 * interface
 *
 * dpwe@media.mit.edu  1994nov25
 */

/*
    THIS CODE SOMETIMES CRASHES - WHAT YOU SHOULD KNOW

    This is obviously hairy stuff because it's operating at interrupt time.
    Doesn't sound that bad I know, but there it is.  My tests reveal that if
    I run this code in a tight loop with aifReadFrames, I occasionally get
    crashes somewhere in the SCSI manager, doing deferred tasks.  I guess you
    could consider this to be a problem with the THINK_C fread implementation;
    I don't know.  Anyway, for data coming from memory, I think it works OK.

    dpwe 1994nov27
 */

#include "dpwelib.h"
/* #include <assert.h> */
/* Mac include .. */
#include <Sound.h>
#define rate44khz ((unsigned long)(44100.0*65536))

/* #define MAIN /* pull in test code at bottom to try and play an AIFF */

/* #define DEBUG /* don't actually call the soundmanager, fake it, print progress */

#undef	DBGFPRINTF
#ifdef DEBUG
#define DBGFPRINTF(a)	fprintf a
#define ASSERT(a)	if(!(a))  { fprintf(stderr, "Assert failed at line %d\n",__LINE__); exit(1); }
#define NOSM	/* don't actually call the sound manager, just run through the motions */
#else /* !DEBUG */
#define DBGFPRINTF(a) /* */
#define ASSERT(a)     /* */
#endif /* DEBUG */

typedef struct {	/* descriptor for audio IO channel */
    int mode;		/* AU_INPUT or AU_OUTPUT */
    int ipchans;	/* chans supplied by caller */
    int opchans;	/* chans we write to OS */
    int ipformat;	/* format of data from caller */
    int opformat;	/* format of data we write to OS */
    SndChannelPtr schn;
    SndCommand	  scmd;
    SndDoubleBufferHeader  dbh;
    SndDoubleBufferPtr	   bufs[2]; /* the two double buffers */
    int currentBuf;	/* our own track of which buf is current */
    int bufFull[2];
    int running;	/* flag as to whether we have started yet */
    long     bufSize;	    /* number of frames allocated per buffer */
    /* data for the callbacks */
    void     *data;	    /* pointer to the base of the sampled data */
    long     totalFrames;   /* how many frames there are */
    long     doneFrames;    /* how many we have already copied */
    /* debug stats */
    int completedblocks;
    int underruns;
} AUSTRUCT;

#define AUSTRUCT_DEFINED    /* stop redefinition in audIO.h */

#include "audIO.h"
#include "sndfutil.h"	/* for Convert* */

static pascal void DoubleBackProc(SndChannelPtr schn, SndDoubleBufferPtr db);

int AUSampleSizeOf(int format)
{
    return format & AU_BYTE_SIZE_MASK;
}

void AUReport(AUSTRUCT *au)
{   /* report internal statistics at end of run */
    DBGFPRINTF((stderr, "AUReport: blocks %d underruns %d\n",
		au->completedblocks, au->underruns));
}

void AUGetStats(AUSTRUCT *au, int *nblocks, int *nunder)
{
    *nblocks = au->completedblocks;  *nunder = au->underruns;
}

#ifdef NOSM
static int dbbuf = 0;	/* next buffer to return as if finished */
static int fakeunderrun = 0;	/* flag to simulate an underrun */

static void ReturnNextBuf(AUSTRUCT *au)
{   /* simulate the interrupt call to DoubleBack returning the next buffer */
    if(! ((au->bufs[0]->dbFlags & dbBufferReady) && (au->bufs[1]->dbFlags & dbBufferReady)) )  {
	DBGFPRINTF((stderr, "RetNext: not both buffers full (%d,%d)\n",
		    au->bufs[0]->dbFlags, au->bufs[1]->dbFlags));
	return;
    } else {
	SndDoubleBufferPtr db = au->bufs[dbbuf];
	db->dbFlags = 0;
	db->dbNumFrames = 0;
	DoubleBackProc(au->schn, db);
	dbbuf = 1-dbbuf;
    }
}

#endif /* NOSM */

static void dbpFill(AUSTRUCT *au)
{   /* using the data pointers in the au, fill in as much more into the current db.
       Carry on and fill the other if it is not already full. */
    char  *src, *dst;
    int   ipchans = au->ipchans;
    int   opchans = au->opchans;
    int   ipfmt = au->ipformat;
    int   opfmt = au->opformat;
    int   ipfmtsz = AUSampleSizeOf(ipfmt);
    int   opfmtsz = AUSampleSizeOf(opfmt);
    long  i, todo;
    int   otherBufFull, thisbuf = au->currentBuf;
    SndDoubleBufferPtr db = au->bufs[thisbuf];

    todo = MIN(au->bufSize - db->dbNumFrames, au->totalFrames - au->doneFrames);
    src = ((char *)au->data) + au->doneFrames * ipchans * ipfmtsz;
    dst = ((char *)db->dbSoundData) + db->dbNumFrames * opchans * opfmtsz;
    CvtChansAndFormat(src, dst, ipchans, opchans, ipfmt, opfmt, todo);
    au->doneFrames  += todo;
    db->dbNumFrames += todo;
    DBGFPRINTF((stderr, "  dbFill(%d) +%ld -> %ld\n", thisbuf, todo,
		db->dbNumFrames));
    /* first check if this buffer is full.  If it is, we need to block until
       the next turnaround, so don't clear the 'ready for more data' semaphore */
    if(db->dbNumFrames == au->bufSize)	{	/* this buffer all set to go */
	au->bufFull[thisbuf] = 1;
	/* once we set this flag, if the interrupt occurs it will automatically
	   call the follow-on routine.	We may try to too, but it will already
	   be full - no harm done. */
	db->dbFlags |= dbBufferReady;	/* triggers the DBP to be called for the other */
	otherBufFull = au->bufFull[1-thisbuf];
#ifdef NOSM
	if(fakeunderrun)  { fakeunderrun=0; ReturnNextBuf(au); }
#endif /* NOSM */
	if(!otherBufFull)  {	/* we have to fill both buffers - like an underrun */
	    ++(au->underruns);
	    au->currentBuf = 1-thisbuf;
	    dbpFill(au);
	}
    } else {	/* we did not fill the buffer - must have run out of data */
	/* mark that we would like more data */
	au->doneFrames	= 0;
	au->totalFrames = 0;
	au->data	= NULL;
    }
}

#include <string.h> /* for memset () */

static pascal void DoubleBackProc(SndChannelPtr schn, SndDoubleBufferPtr db)
{   /* callback when one buffer is empty */
    AUSTRUCT *au = (AUSTRUCT *)db->dbUserInfo[0];
    int returnedbuf = (db==au->bufs[0])?0:1;

    ++(au->completedblocks);
    DBGFPRINTF((stderr, "DoubBack: %d returned, underrun=%d\n", returnedbuf, underrun));
    /* flag that returned buf is ready for filling */
    db->dbNumFrames = 0;
    au->bufFull[returnedbuf] = 0;
    /* since this routine isn't called until both buffers are marked ready,
       we know that the other buffer has completed filling, so we can go right
       ahead and try to fill this buffer. */
    if(au->bufFull[1-returnedbuf]) {
	au->currentBuf = returnedbuf;
	dbpFill(au);
    } else {
	/* underrun, somehow */
	++(au->underruns);
    }
    DBGFPRINTF((stderr, "DoubBack done\n"));
}

static int MySndIdle(SndChannelPtr schn)
{   /* an activity to put in a polling loop */
#ifndef NOSM
    SCStatus	status;
    SndChannelStatus(schn, sizeof(status), &status);
    return status.scChannelBusy;
#else  /* !NOSM */
    ReturnNextBuf(au);
    return au->doneFrames < au->totalFrames;	/* not right; hack */
#endif /* NOSM */
}

static void AUWaitNoData(AUSTRUCT *au)
{   /* Wait until the double-back procedure indicates that the current data
       has all been copied into buffers. */
    while(au->data != NULL)  {
	MySndIdle(au->schn);
    }
}

static void AUWaitNotBusy(AUSTRUCT *au)
{   /* block until the current playing is complete */
    /* can look at channel to see if it's busy, also look at double buffers
       to see when they have reached the end.  We'll look at the channel */
    while(MySndIdle(au->schn))
	;
    au->running = 0;
}

static void AUNewData(AUSTRUCT *au, void *buf, long frames)
{   /* queue up some new data */
    DBGFPRINTF((stderr, "AUNewData: %ld frames\n", frames));
    AUWaitNoData(au);
    au->data = buf;
    au->totalFrames = frames;
    dbpFill(au);
    DBGFPRINTF((stderr, "AUNewData done : curbuf %d, full(%d,%d)\n",
		au->currentBuf, au->bufFull[0], au->bufFull[1]));
}

static int doStereo = 1;
static int do16bit  = 1;

#define DFLT_BUFTIME (0.25)	/* default buffering is 1/4 sec */

AUSTRUCT *AUOpen(int format, double srate, int chans, long desbuf, int mode)
{   /* Open a new audio IO channel.
       Define data format, #channels and sampling rate.
       Request that output buffer be desbuf frames long (0 gives default).
       <mode> is AIO_INPUT or AIO_OUTPUT  */
    AUSTRUCT *au;
    int ipchans;
    int opchans;
    int ipfmt;
    int opfmt;
    int ipfmtsz;
    int opfmtsz;
    long dbbytes;
    int oerr = 0, i;
    unsigned long fxSr = (unsigned long)(srate * (1L<<16));

    if((au = TMMALLOC(AUSTRUCT, 1, "AUOpen"))==NULL)  {
	DBGFPRINTF((stderr, "AUOpen : no memory for au\n"));
	return NULL;
    }

    /* 'pull in' to hardware supported sample rates to avoid poor
       interpolation (no 'frac' continuity across blocks) */
    if(42000 < srate && srate < 46000)	     fxSr = rate44khz;
    else if(21000 < srate && srate < 23000)  fxSr = rate22khz;
    else if(10500 < srate && srate < 11500)  fxSr = rate11khz;	

    ipchans = au->ipchans = chans;
    opchans = au->opchans = doStereo?chans:1;
    ipfmt   = au->ipformat = format;
    opfmt   = au->opformat = do16bit?AFMT_SHORT:AFMT_CHAR_OFFS;
    ipfmtsz = AUSampleSizeOf(ipfmt);
    opfmtsz = AUSampleSizeOf(opfmt);

    if(mode == AIO_INPUT)  {
	DBGFPRINTF((stderr, "no input on mac yet\n"));
	TMFREE(au, "input au");
	return NULL;
    }
    /* assume just output now */
    if(desbuf == 0)  {
	desbuf = srate*DFLT_BUFTIME;
    }
    /* allocate double buffer handles */
    dbbytes = sizeof(SndDoubleBuffer) + desbuf*opfmtsz*opchans;
    for(i = 0; i<2; ++i)  {
	au->bufs[i] = (SndDoubleBuffer *)TMMALLOC(char, dbbytes, "dbuf01");
	if(au->bufs[i] == NULL)  {
	    DBGFPRINTF((stderr, "AUOpen: no memory for buffers\n"));
	    TMFREE(au, "au");
	    return NULL;
	}
	au->bufs[i]->dbNumFrames = 0;
	au->bufs[i]->dbFlags = 0;
	au->bufs[i]->dbUserInfo[0] = (long)(au);
	au->dbh.dbhBufferPtr[i] = au->bufs[i];
    }

    /* initialize sound double buffer header */
    au->dbh.dbhNumChannels = opchans;
    au->dbh.dbhSampleSize  = 8 * opfmtsz;
    au->dbh.dbhCompressionID = 0;
    au->dbh.dbhPacketSize    = 0;
    au->dbh.dbhSampleRate    = fxSr;
    au->dbh.dbhDoubleBack    = (SndDoubleBackProcPtr)DoubleBackProc;

    /* setup the DB struct */
    au->data = NULL;
    au->totalFrames = 0;    /* how many frames there are */
    au->doneFrames  = 0;    /* how many we have already copied */
    au->bufSize = desbuf;   /* number of frames allocated per buffer */

    au->currentBuf = 0;
    au->running = 0;
    au->bufFull[0] = au->bufFull[1] = 0;    /* our underrun semaphores */

    /* debug stats */
    au->completedblocks = 0;
    au->underruns	= -1;	/* first call adds a spurious underrun pre-filling buffers */

    /* setup sound output hardware */
    au->schn = NULL;
#ifndef NOSM
    if(opchans == 1)
	oerr = SndNewChannel(&(au->schn), sampledSynth, initMono, NULL);
    else /* opchans == 2 */
	oerr = SndNewChannel(&(au->schn), sampledSynth, initStereo, NULL);
#endif /* NOSM */
    if(oerr)  {
	TMFREE(au->bufs[1], "bufs1");
	TMFREE(au->bufs[0], "bufs0");
	TMFREE(au, "au");
	return NULL;	    /* fail */
    }
    return au;
}

static long AUStart(AUSTRUCT *au, void *buf, long frames)
{   /* Have not actually started the double buffering yet - wait for
       both bufs to fill */
    ASSERT(au->data == NULL);	    /* if we really haven't started, but there is
				       waiting data, AUNewData will block forever */
    AUNewData(au, buf, frames);
    if(au->bufFull[1])	{	    /* second buffer is full - OK to start */
	DBGFPRINTF((stderr, "AUStart: starting\n"));
	/* start the command */
#ifndef NOSM
	SndPlayDoubleBuffer(au->schn, &(au->dbh));
#endif /* NOSM */
	au->running = 1;
    }
    return frames;  /* does not block */
}

long AUWrite(AUSTRUCT *au, void *buf, long frames)
{   /* write some sound to the open audio IO channel, per description */
    /* wait for existing double buffering to complete */
    if(!au->running)  {
	AUStart(au, buf, frames);
    } else {
	AUNewData(au, buf, frames);
    }
    /* we can't return until this data is all used up - else collision */
    while(au->data && au->doneFrames < au->totalFrames)  {
	MySndIdle(au->schn);
    }
}

long AURead(AUSTRUCT *auch, void *buf, long frames)
{   /* read some sound from the open audio IO channel, per description */
    return 0;	/* not implemented */
}

long AUBufConts(AUSTRUCT *au)
{   /* see how many sample frames are currently in audio buffer.
       (can be used for synchronization, timing etc). */
    return au->totalFrames - au->doneFrames;
}

void AUClose(AUSTRUCT *au, int now)
{   /* close an audio IO channel.  <now> nonzero kills play immediately */
    /* flag the interrupt routine to stop at the next buffer */
    if(!now)  {
	AUWaitNoData(au);   /* use up the current buffers */
	/* mark the last, partially-full buffer as the last */
	au->bufs[au->currentBuf]->dbFlags |= (dbBufferReady|dbLastBuffer);
	if(!au->running)  {
	    au->bufFull [0] = au->bufFull[1] = 1;   /* trick AUStart into starting */
	    AUStart(au, NULL, 0);
	}
	AUWaitNotBusy(au);  /* wait for it to be played */
    } else {
	au->scmd.cmd = quietCmd;
	au->scmd.param1 = 0;
	au->scmd.param2 = 0;
#ifndef NOSM
	SndDoImmediate(au->schn, &(au->scmd));
#endif /* NOSM */
    }
#ifndef NOSM
    SndDisposeChannel(au->schn, FALSE); /* doesn't wait for commands to end */
#endif /* NOSM */
    /* free all the data */
/*  AUReport(au);   */
    TMFREE(au->bufs[1], "bufs1");
    TMFREE(au->bufs[0], "bufs0");
    TMFREE(au, "au");
}

#ifdef MAIN	/* TEST CODE below */
/*
 * macplay.c
 *
 * main routine to play sounds
 * by connecting aifif to audIO
 *
 * dpwe@media.mit.edu 1994nov26
 */

#include "dpwelib.h"
#include <assert.h>
#include "aifif.h"
/* #include "audIO.h" */

#ifdef THINK_C
#include <tc_main.h>
#endif /* THINK_C */

char *programName;

main(argc, argv)
int argc;
char *argv[];
{
    int i;
    char *inpath;
    AIF_STRUCT *aifs;
    AUSTRUCT  *au;
    long pvb[16], pvl;
    short *buf, *buf2=NULL, *bptr;
    long  rem, this, got;
    int   sampsize, chans;
    long  nframes;
    double srate;
    int count = 0;	
    long  bufsize = 32768;
    long  sndbufsize = 8192;

    programName = argv[0];
    if(argc!=2)  {
	fprintf(stderr, "usage: %s aiffile\n", programName);
	exit(-1);
    }
    inpath = argv[1];

    aifs = aifNew();
    if( aifOpenRead(aifs, inpath) )  {
	fprintf(stderr, "%s: could not open %s as sound\n", programName, inpath);
	exit(-1);
    }
    /* query soundfile */
    pvl = 0;
    pvb[pvl++] = AIF_P_SAMPSIZE;   ++pvl;
    pvb[pvl++] = AIF_P_SAMPRATE;   ++pvl;
    pvb[pvl++] = AIF_P_CHANNELS;   ++pvl;
    pvb[pvl++] = AIF_P_NFRAMES;    ++pvl;
    aifGetParams(aifs, pvb, pvl);
    sampsize = pvb[1];
    srate    = FLOATofLONG(pvb[3]);
    chans    = pvb[5];
    nframes  = pvb[7];

    fprintf(stderr, "file %s: sampsz %d, srate %.0f, chans %d, frames %ld\n",
	    inpath, sampsize, srate, chans, nframes);

    assert(sampsize == 16);

    if( (au = AUOpen(AFMT_SHORT, srate, chans, sndbufsize, AIO_OUTPUT))==NULL)	{
	fprintf(stderr, "%s: could not open sound output\n", programName);
	aifClose(aifs);
	exit(-1);
    }
    fprintf(stderr, "Sound buffer size is %ld frames\n", sndbufsize);
    buf = TMMALLOC(short, bufsize*chans, "playbuf");
#define FRAMEY /* */
#ifdef FRAMEY
    fprintf(stderr, "On the fly read-from-file, write in blocks of %ld\n", bufsize);
    rem = nframes;
    got = -1;
    while(got && rem)  {
	this = MIN(rem, bufsize);
	got  = aifReadFrames(aifs, buf, this);
	AUWrite(au, buf, got);
	rem -= got;
	++count;
    }
#else /* not FRAMEY, read whole sound at once */
    buf2 = TMMALLOC(short, nframes*chans, "playbuf");
    if(buf2)  {
	got  = aifReadFrames(aifs, buf2, nframes);
	fprintf(stderr, "Read all %ld frames\n", got);
#define ABITFRAMEY   /* */
#ifdef ABITFRAMEY
	fprintf(stderr, "Passing to AUWrite in blocks of %ld\n", bufsize);
	rem = nframes;	got = -1;
	bptr = buf2;
	while(got && rem)  {
	    this = MIN(rem, bufsize);
	    got = this;  BlockMove(bptr, buf, this*chans*sizeof(short));
	    AUWrite(au, buf, got);
	    rem -= got;
	    bptr += got*chans;
	    ++count;
	}
#else /* not ABITFRAMEY */
	fprintf(stderr, "Giving AUWrite all %ld\n", got);
	AUWrite(au, buf2, got);
#endif /* ABITFRAMEY */
    }
#endif /* FRAMEY */
    fprintf(stderr, "%d calls\n", count);
    AUClose(au, 0);
    TMFREE(buf, "playbuf");
    if(buf2)	TMFREE(buf2, "buf2");
    aifClose(aifs);
    exit(0);
}

#endif /* MAIN */
